import 'package:http/http.dart' as http;
import 'response.dart';
import '../common/default_value.dart';
import '../common/debug.dart';
import '../setup/setup.dart';
import '../model/user.dart';

/// Base api for leancloud
class BaseApi {
  /// http request by Get
  static Future<LeancloudResponse> get(String path, [Map params = null]) async {
    Debug.begin();
    final uri = new Uri.https(DefaultValue.urlAddressCN, path, params);
    Debug.log('GET -> $uri');
    final headers = {
      "Content-Type": "application/json",
      "X-LC-Id": LeancloudInit().getLeancloudId(),
      "X-LC-Sign": LeancloudInit().getSignToken(),
    };
    if (AVUser.currentUser != null) {
      headers.addAll({"X-LC-Session": AVUser.currentUser.getSessionToken()});
    }

    final response = await http.get(uri, headers: headers);
    LeancloudResponse result = LeancloudResponse(response: response);
    Debug.end();
    return result;
  }

  /// http request by Post
  static Future<LeancloudResponse> post(String url, dynamic params) async {
    Debug.begin();
    final uri = new Uri.https(DefaultValue.urlAddressCN, url, null);
    Debug.log('POST -> $uri');
    final headers = {
      "Content-Type": "application/json",
      "X-LC-Id": LeancloudInit().getLeancloudId(),
      "X-LC-Sign": LeancloudInit().getSignToken(),
    };
    if (AVUser.currentUser != null) {
      headers.addAll({"X-LC-Session": AVUser.currentUser.getSessionToken()});
    }
    Debug.log('HEADER -> $headers');
    Debug.log('PARAMS -> $params');
    var result = await http.post(uri, headers: headers, body: params);
    LeancloudResponse leancloudResponse = LeancloudResponse(response: result);
    Debug.end();
    return leancloudResponse;
  }

  /// http request by Put
  static Future<LeancloudResponse> put(
      String path, dynamic params) async {
    Debug.begin();
    final uri = new Uri.https(DefaultValue.urlAddressCN, path, null);
    Debug.log('PUT -> $uri');
    final headers = {
      "Content-Type": "application/json",
      "X-LC-Id": LeancloudInit().getLeancloudId(),
      "X-LC-Sign": LeancloudInit().getSignToken(),
    };
    if (AVUser.currentUser != null) {
      headers.addAll({"X-LC-Session": AVUser.currentUser.getSessionToken()});
    }
    Debug.log('HEADER -> $headers');
    Debug.log('PARAMS -> $params');
    final response = await http.put(uri, headers: headers, body: params);
    LeancloudResponse result = LeancloudResponse(response: response);
    return result;
  }

  /// http request by Delete
  static Future<LeancloudResponse> delete(String url) async {
    Debug.begin();
    final uri = new Uri.https(DefaultValue.urlAddressCN, url, null);
    Debug.log('DELETE -> $uri');
    final headers = {
      "Content-Type": "application/json",
      "X-LC-Id": LeancloudInit().getLeancloudId(),
      "X-LC-Sign": LeancloudInit().getSignToken(),
    };
    if (AVUser.currentUser != null) {
      headers.addAll({"X-LC-Session": AVUser.currentUser.getSessionToken()});
    }
    Debug.log('HEADER -> $headers');
    var result = await http.delete(uri, headers: headers);
    LeancloudResponse leancloudResponse = LeancloudResponse(response: result);
    Debug.end();
    return leancloudResponse;
  }

}