import 'base_api.dart';
import 'response.dart';
import '../common/default_value.dart';
import '../model/object.dart';
import '../common/constraint.dart';

/// AVObject rest api
class ObjectApi {
  /// Create Object
  Future<LeancloudResponse> createObject(AVObject object) async {
    String url = DefaultValue.apiVersion + UrlValue.objectCreate;
    url = url.replaceAll(DefaultValue.classNameForReplace, object.getClassName());
    String params = object.getFieldsJson();

    return BaseApi.post(url, params).then((result) {
      return result;
    });
  }

  /// get object by Id
  /// Using this function, please set all of them:
  ///  * AVObject className
  ///  * AVObject Id
  Future<LeancloudResponse> getObjectById(AVObject object) async {
    String url = DefaultValue.apiVersion + UrlValue.objectGetById;
    url = url.replaceAll(DefaultValue.classNameForReplace, object.getClassName());
    url = url.replaceAll(DefaultValue.objectIdForReplace, object.getObjectId());

    return BaseApi.get(url).then((result) {
      return result;
    });
  }

  /// update object by id
  /// Using this function, please set all of them:
  ///  * AVObject className
  ///  * AVObject Id
  Future<LeancloudResponse> updateObjectById(AVObject object) async {
    String url = DefaultValue.apiVersion + UrlValue.objectUpdateById;
    url = url.replaceAll(DefaultValue.classNameForReplace, object.getClassName());
    url = url.replaceAll(DefaultValue.objectIdForReplace, object.getObjectId());
    String params = object.getUpdateFieldsJson();

    return BaseApi.put(url, params).then((result) {
      return result;
    });
  }

  /// get objects or by constraint, default limit is 100
  /// Using this function, please set:
  ///  * AVObject className
  Future<LeancloudResponse> getObjects(AVObject object, [Constraint constraint = null]) async {
    String url = DefaultValue.apiVersion + UrlValue.objectGet;
    url = url.replaceAll(DefaultValue.classNameForReplace, object.getClassName());

    var params = constraint == null ? null : constraint.getConstraint();
    return BaseApi.get(url, params).then((result) {
      return result;
    });
  }

  /// delete object by id
  /// Using this function, please set all of them:
  ///  * AVObject className
  ///  * AVObject Id
  Future<LeancloudResponse> deleteObjectById(AVObject object) async {
    String url = DefaultValue.apiVersion + UrlValue.objectDeleteById;
    url = url.replaceAll(DefaultValue.classNameForReplace, object.getClassName());
    url = url.replaceAll(DefaultValue.objectIdForReplace, object.getObjectId());

    return BaseApi.delete(url).then((result) {
      return result;
    });
  }
}