import 'base_api.dart';
import 'response.dart';
import '../common/default_value.dart';
import '../model/user.dart';

/// AVUser rest api
class UserApi {
  /// Sign up
  Future<LeancloudResponse> signUp(AVUser user) async {
    String url = DefaultValue.apiVersion + UrlValue.usersSignUp;
    String params = user.getFieldsJson();

    return BaseApi.post(url, params).then((result) {
      return result;
    });
  }

  /// Update User by Id
  /// Using this function, please set
  ///  * AVUser.currentUser first.
  Future<LeancloudResponse> updateById(AVUser user) async {
    String url = DefaultValue.apiVersion + UrlValue.usersUpdateById;
    url = url.replaceAll(DefaultValue.objectIdForReplace, user.getObjectId());
    String params = user.getUpdateFieldsJson();

    return BaseApi.put(url, params).then((result) {
      return result;
    });
  }

  /// Update UserPassword
  /// Using this function, please set all of them:
  ///   * AVUser.currentUser
  ///   * AVUser.setPassword
  ///   * AVUser.setNewPassword
  Future<LeancloudResponse> updateUserPassword(AVUser user) async {
    String url = DefaultValue.apiVersion + UrlValue.usersUpdatePasswordById;
    url = url.replaceAll(DefaultValue.objectIdForReplace, user.getObjectId());
    String params = '{"old_password":"${user.getPassword()}", "new_password":"${user.getNewPassword()}"}';

    return BaseApi.put(url, params).then((result) {
      return result;
    });
  }

  /// Refresh SessionToken
  /// Using this function, please set
  ///   * AVUser.currentUser
  Future<LeancloudResponse> refreshSessionToken() async {
    String url = DefaultValue.apiVersion + UrlValue.usersRefreshSessionTokenById;
    url = url.replaceAll(DefaultValue.objectIdForReplace, AVUser.currentUser.getObjectId());

    return BaseApi.put(url, null).then((result) {
      return result;
    });
  }


  /// Delete User by Id
  /// Using this function
  ///   * please set AVUser.currentUser first. (recommend)
  ///   * setup MasterKey.
  Future<LeancloudResponse> deleteUserById(AVUser user) async {
    String url = DefaultValue.apiVersion + UrlValue.usersDeleteById;
    url = url.replaceAll(DefaultValue.objectIdForReplace, user.getObjectId());

    return BaseApi.delete(url).then((result) {
      return result;
    });
  }
}