import 'dart:convert';

class AVObject {
  /// private property for Class Name
  var _className;
  /// private property for Class Fields
  var _fields = {};

  /// Create an AVObject with Class Name
  AVObject(this._className);

  /// Add or Update field value with [value] into this Object by [key]
  void put(String key, Object value) {
    this._fields.addAll({key: value});
  }

  /// Get [key] field value
  dynamic get(String key) {
    return this._fields[key];
  }

  /// Get Class Name
  String getClassName() {
    return _className;
  }

  /// update entity by response body
  void updateByLeancloudResponseBody(String body) {
    var queriedFields = jsonDecode(body);
    queriedFields.forEach((key, value) {
      if (key == "className") this._className = value;
      this.put(key, value);
    });
  }

  /// Get this Object Id
  String getObjectId() {
    return this._fields["objectId"];
  }

  /// Overwritten toString() function, to return this object's JSON string.
  String toString() {
    if (this._fields.isEmpty) {
      throw Exception("Empty field, before save or create, you must to add field! Using put() function.");
    }
    var fieldsString = jsonEncode(this._fields);
    var object = {};
    object.addAll({"className": this._className});
    object.addAll({"fields": fieldsString});
    return jsonEncode(object);
  }

  /// Get Fields Json
  String getFieldsJson() {
    return jsonEncode(this._fields);
  }

  /// Get Fields Json without objectId/sessionToken/password/updatedAt/createdAt/emailVerified/mobilePhoneVerified
  String getUpdateFieldsJson() {
    var fields = Map.from(this._fields);
    fields.remove("objectId");
    fields.remove("sessionToken");
    fields.remove("password");
    fields.remove("updatedAt");
    fields.remove("createdAt");
    fields.remove("emailVerified");
    fields.remove("mobilePhoneVerified");
    return jsonEncode(fields);
  }
}