import 'dart:convert';
import 'package:convert/convert.dart';
import 'package:crypto/crypto.dart';

/// Init Leancloud
class LeancloudInit {

  factory LeancloudInit() =>_getInstance();
  static LeancloudInit get instance => _getInstance();
  static LeancloudInit _instance;
  LeancloudInit._internal() {
    // init
  }

  static LeancloudInit _getInstance() {
    if (_instance == null) {
      _instance = new LeancloudInit._internal();
    }
    return _instance;
  }

  /// Set leancloudId
  void setLeanCloudId(String id) {
    _leancloudId = id;
  }

  /// Set leancloudKey
  void setLeancloudKey(String key) {
    _leancloudKey = key;
  }

  /// Set masterKey
  /// If set masterKey, masterKey will replace leancloudKey for request api.
  void setMasterKey(String key) {
    _masterKey = key;
  }

  /// Set isDebug. default is false
  void setDebug(bool debug) {
    _isDebug = debug;
  }

  /// Get is debug.
  bool isDebug() {
    return _isDebug;
  }

  /// Get LeancloudId
  String getLeancloudId() {
    return _leancloudId;
  }

  /// Using for http header X-LC-Sign value
  String getSignToken() {
    var key = _leancloudKey;
    String timestamp = DateTime.now().millisecondsSinceEpoch.toString();
    String appendString = '';
    if (_masterKey != null && _masterKey != '') {
      key = _masterKey;
      appendString = ',master';
    }
    String md5 = _generateMd5(timestamp + key);
    String signToken = md5 + ',' + timestamp + appendString;
    return signToken;
  }

  String _leancloudId = '';
  String _leancloudKey = '';
  String _masterKey = '';
  bool _isDebug = false;

  /// md5
  String _generateMd5(String data) {
    var content = new Utf8Encoder().convert(data);
    var digest = md5.convert(content);
    return hex.encode(digest.bytes);
  }
}