import 'package:flutter_test/flutter_test.dart';

import 'package:leancloud_plugin/leancloud_plugin.dart';

void main() {
  group('user test', () {
    final userApi = UserApi();
    var user = AVUser();
    String userName = DateTime.now().millisecondsSinceEpoch.toString();

    setUp(() {
      final leancloud = LeancloudInit();
      leancloud.setLeanCloudId('KT7xNserhsIzV08m9XWrOJax-gzGzoHsz');
      leancloud.setLeancloudKey('yLtdsHGRwqi3TpsMGK2pKjGE');
      leancloud.setDebug(true);
    });

    test('SignUp test', () async {
      user.setUsername(userName);
      user.setPassword("maomishen");
      user.put("customField", "customFieldValue");

      var f1 = userApi.signUp(user);
      expect(await f1, isNotNull);

      f1.then((value) {
        print(value.body);
        expect(value.jsonResult['username'], userName);
        user.updateByLeancloudResponseBody(value.body);
        AVUser.currentUser = user;
      });
    });

    test('Update user test', () async {
      user.put("customField", "customFieldValue changed!");

      var f1 = userApi.updateById(user);
      expect(await f1, isNotNull);

      f1.then((value) {
        print(value.body);
        expect(value.jsonResult['objectId'], user.getObjectId());
        user.updateByLeancloudResponseBody(value.body);
      });
    });

    test('Update user password test', () async {
      user.setNewPassword("maomishen2");

      var f1 = userApi.updateUserPassword(user);
      expect(await f1, isNotNull);

      f1.then((value) {
        print(value.body);
        expect(value.jsonResult['objectId'], user.getObjectId());
        user.updateByLeancloudResponseBody(value.body);
      });
    });

    test('refresh user SessionToken test', () async {
      user.setNewPassword("maomishen2");

      var f1 = userApi.refreshSessionToken();
      expect(await f1, isNotNull);

      f1.then((value) {
        print(value.body);
        expect(value.jsonResult['objectId'], user.getObjectId());
        user.updateByLeancloudResponseBody(value.body);
      });
    });

    test('Update user again test', () async {
      user.put("customField", "customFieldValue changed again!");

      var f1 = userApi.updateById(user);
      expect(await f1, isNotNull);

      f1.then((value) {
        print(value.body);
        expect(value.jsonResult['objectId'], user.getObjectId());
        user.updateByLeancloudResponseBody(value.body);
      });
    });

    test('Delete User test', () async {
      var f1 = userApi.deleteUserById(user);
      expect(await f1, isNotNull);

      f1.then((value) {
        print(value.body);
      });
    });

  });

  group('object test', ()
  {
    final objectApi = ObjectApi();
    var object = AVObject("test_api");

    setUp(() {
      final leancloud = LeancloudInit();
      leancloud.setLeanCloudId('KT7xNserhsIzV08m9XWrOJax-gzGzoHsz');
      leancloud.setLeancloudKey('yLtdsHGRwqi3TpsMGK2pKjGE');
      leancloud.setDebug(true);
    });

    test('Create object test', () async {
      object.put("testField", "this is String value");

      var f1 = objectApi.createObject(object);
      expect(await f1, isNotNull);

      f1.then((value) {
        print(value.body);
        object.updateByLeancloudResponseBody(value.body);
      });
    });

    test('Get object by Id test', () async {
      var f1 = objectApi.getObjectById(object);
      expect(await f1, isNotNull);

      f1.then((value) {
        print(value.body);
        expect(value.jsonResult['testField'], "this is String value");
        object.updateByLeancloudResponseBody(value.body);
      });
    });

    test('Update object by Id test', () async {
      object.put("testField", "this is String value changed!");

      var f1 = objectApi.updateObjectById(object);
      expect(await f1, isNotNull);

      f1.then((value) {
        print(value.body);
        object.updateByLeancloudResponseBody(value.body);
      });
    });

    test('Get objects test', () async {
      object.put("testField", "this is String value changed!");

      var f1 = objectApi.getObjects(object);
      expect(await f1, isNotNull);

      f1.then((value) {
        print(value.body);
        object.updateByLeancloudResponseBody(value.body);
      });
    });

    test('Get objects by constraints test', () async {
      Constraint constraint = Constraint();
      constraint.setLimit(10);

      var f1 = objectApi.getObjects(object, constraint);
      expect(await f1, isNotNull);

      f1.then((value) {
        print(value.body);
        object.updateByLeancloudResponseBody(value.body);
      });
    });

    test('Delete object by Id test', () async {
      var f1 = objectApi.deleteObjectById(object);
      expect(await f1, isNotNull);

      f1.then((value) {
        print(value.body);
      });
    });
  });
}
